/*
#******[ sub:functions:inet_address stuff ]*******************
#
#  Target    : inet/address manipulation/conversion functions
#
#  Platforms : Linux,BSD,Solaris (mostly OS independent)
#  Copyright : (c) 1996-2001 SysMan
#  License   : GNU License
#  Author    : Alex Petrov sysman@sysman.net
#
#************************************************************
 (c) 1996-2001 by Alex Petrov aka SysMan
   e-mail: sysman@sysman.net
     http://www.sysman.net/linux/sub/
     
# $Id: addr_func.c,v 1.2 2001/08/31 11:53:59 sysman Exp $
*/
#include <stdio.h>
#include <string.h>
#include <unistd.h>

/* ***** variables ***** */

char ret_addr[16];

union addr_type
{ 
unsigned long l;
unsigned char b[3];
} adr;

/* Functions predifinition */

unsigned long str2addr(char *str);
char *addr2str(unsigned long addr);
int addr_in_snet(unsigned long addr,unsigned long snet,unsigned long mask);

/*  -----------------------------------------------
   *** str2addr - convert string to address (ul) ***
    -----------------------------------------------
 work like inet_addr/inet_aton, but byte order is normal
 first come first, parse short strings like usual
 127.1 <=> 127.0.0.1
 127.2.1 <=> 127.2.0.1
 RETURN: NULL if fail , otherwise (unsigned long) address.
*/
unsigned long str2addr(char *str)
{
char in_str[18];
char *ptr;
int i=0,p=0;

strncpy(in_str,str,16);

 /*  Fast check only numbaz & dots  */
 while(in_str[i]!=0)
 {
 if ( in_str[i]>'9' || in_str[i]<'0' )
  if ( in_str[i]!='.' ) return 0; // fail
  else { p++; }
 i++;
 }

 if(p>3) return 0; // Fail , to many dots
 
 adr.l=0;       // clean out space before use

 ptr=strtok((char *)&in_str[0],".");
 if(ptr)
   { adr.b[3] = strtoul(ptr,0,10);
     if(adr.b[3]>255) return 0; // fail
   }

 if(p>=2)
 {
  ptr=strtok((char )0,".");
  if(ptr)
  { adr.b[2] = strtoul(ptr,0,10);
   if(adr.b[2]>255) return 0; // fail
  }
 }

 if(p==3)
 {
  ptr=strtok((char )0,".");
  if(ptr)
  { adr.b[1] = strtoul(ptr,0,10);
   if(adr.b[1]>255) return 0; // fail
  }
 }
if(p>1)
 {
  ptr=strtok((char )0,".");
  if(ptr)
  { adr.b[0] = strtoul(ptr,0,10);
    if(adr.b[0]>255) return 0; // fail
  }
 }
return (adr.l); // return OK addr
}
/*   -----------------------------------------------
    *** addr2str - convert (ul) address to string ***
     -----------------------------------------------
*/
char *addr2str(unsigned long addr)
{
adr.l=addr;
sprintf(ret_addr,"%u.%u.%u.%u",adr.b[3],adr.b[2],adr.b[1],adr.b[0]);
return (ret_addr);
}

/*  ------------------------------------------- 
   *** addr_in_snet - check if address(ul)   ***
   ***   in subnet/mask , 1 - yes/0 - fail   ***
    -------------------------------------------
*/
int addr_in_snet(unsigned long addr,unsigned long snet,unsigned long mask)
{
unsigned long max_addr;
max_addr=(snet+(0xffffffff-mask));
/*   printf(" min %s",addr2str(snet));
   printf(" max %s",addr2str(max_addr));
   printf(" addr=%s\n",addr2str(addr)); */
//   printf(" %lu %lu %lu\n",addr,snet,max_addr); 
if(addr<=max_addr && addr>=snet) return 1;
else return 0;  // fail
}
/*
    inet_address conversion functions 
    Writed by Alex Petrov aka SysMan, 1996
EOF
*/